/**
 * Copyright (C) 2020 leoetlino
 *
 * This file is part of oead.
 *
 * oead is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * oead is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with oead.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <pybind11/pybind11.h>

#include "pybind11_common.h"

namespace py = pybind11;

namespace oead::bind {

void BindAamp(py::module& m);
void BindByml(py::module& m);
void BindCommonTypes(py::module& m);
void BindGsheet(py::module& m);
void BindSarc(py::module& m);
void BindYaz0(py::module& m);

}  // namespace oead::bind
